/*****************************************************************************
 *  Rohde&Schwarz DC power supplies instrument driver
 *
 *  VXI Plug&Play Instrument Driver
 *  Original Release: May 2014
 *  By: Lukas Hynek
 *****************************************************************************/

#ifndef __RSHMC804X_HEADER
#define __RSHMC804X_HEADER

#include "rscore.h"
#include "rshmc804x_attributes.h"
#include "rshmc804x_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSHMC804X_CLASS_DRIVER_DESCRIPTION           ""
#define RSHMC804X_CLASS_DRIVER_PREFIX                ""
#define RSHMC804X_CLASS_DRIVER_VENDOR                ""
#define RSHMC804X_CLASS_DRIVER_REVISION              ""
#define RSHMC804X_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSHMC804X_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSHMC804X_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSHMC804X_SPECIFIC_DRIVER_PREFIX             "RSHMC804X"
#define RSHMC804X_SPECIFIC_DRIVER_LOCATOR            ""
#define RSHMC804X_SPECIFIC_DRIVER_VENDOR             ""
#define RSHMC804X_MAJOR_VERSION                      2L    /* Instrument driver major version          */
#define RSHMC804X_MINOR_VERSION                      1L    /* Instrument driver minor version          */
#define RSHMC804X_MINOR_MINOR_VERSION                1L    /* Instrument driver minor minor version    */

/* Driver Capabilities */

#define RSHMC804X_SUPPORTED_INSTRUMENT_MODELS        "HMC8043,HMC8042,HMC8041,NGE102,NGE103,NGA101,NGA141,NGA102,NGA142,NGF201"    /* Instrument driver supported model(s)     */
#define RSHMC804X_GROUP_CAPABILITIES                 ""
#define RSHMC804X_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *------------------------------ Useful Macros -----------------------------*
 ****************************************************************************/

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/

#define RSHMC804X_SIMULATION_ID_QUERY      "Rohde&Schwarz,HMC8043,123456/789,1.40"  /* Default definition of ID Query Response for simulation */
#define RSHMC804X_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSHMC804X_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSHMC804X_OPC_TIMEOUT              10000L   /* Maximum time to wait for OPC in milliseconds */

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSHMC804X_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSHMC804X_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSHMC804X_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSHMC804X_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSHMC804X_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSHMC804X_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSHMC804X_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSHMC804X_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSHMC804X_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSHMC804X_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSHMC804X_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSHMC804X_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSHMC804X_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSHMC804X_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSHMC804X_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSHMC804X_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSHMC804X_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSHMC804X_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSHMC804X_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSHMC804X_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSHMC804X_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSHMC804X_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSHMC804X_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSHMC804X_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSHMC804X_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSHMC804X_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSHMC804X_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSHMC804X_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSHMC804X_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSHMC804X_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSHMC804X_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME                                    /* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

/* BEGIN GENERATE */
#define RSHMC804X_ATTR_DISPLAY_CLEAR                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 104
#define RSHMC804X_ATTR_DISPLAY_TEXT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 105
#define RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 88
#define RSHMC804X_ATTR_OUTPUTS_CURRENT_LIMIT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 86
#define RSHMC804X_ATTR_OUTPUTS_CURRENT_STEP_SIZE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 146
#define RSHMC804X_ATTR_OUTPUTS_RESET_CURRENT_STEP_SIZE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 147
#define RSHMC804X_ATTR_OUTPUTS_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 87
#define RSHMC804X_ATTR_OUTPUTS_MASTER_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 107
#define RSHMC804X_ATTR_CHANNEL_ONLY_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 108
#define RSHMC804X_ATTR_OUTPUTS_VOLTAGE_LEVEL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 95
#define RSHMC804X_ATTR_OUTPUTS_VOLTAGE_STEP_SIZE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 148
#define RSHMC804X_ATTR_OUTPUTS_RESET_VOLTAGE_STEP_SIZE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 149
#define RSHMC804X_ATTR_CHANNEL_FUSION                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 203
#define RSHMC804X_ATTR_MODE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 204
#define RSHMC804X_ATTR_QUERY_MODE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 205
#define RSHMC804X_ATTR_VOLTMETER_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 206
#define RSHMC804X_ATTR_QUERY_VOLTMETER_VALUE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 207
#define RSHMC804X_ATTR_OUTPUTS_ARBITRARY_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 121
#define RSHMC804X_ATTR_OUTPUTS_ARBITRARY_REPETITIONS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 122
#define RSHMC804X_ATTR_OUTPUTS_ARBITRARY_ENDPOINT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 123
#define RSHMC804X_ATTR_OUTPUTS_ARBITRARY_RESET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 124
#define RSHMC804X_ATTR_OUTPUTS_ARBITRARY_SAVE_FILE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 125
#define RSHMC804X_ATTR_OUTPUTS_ARBITRARY_LOAD_FILE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 126
#define RSHMC804X_ATTR_OUTPUTS_ARBITRARY_TRIGGERED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 151
#define RSHMC804X_ATTR_OUTPUTS_ARBITRARY_TRIGGER_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 152
#define RSHMC804X_ATTR_OUTPUTS_FUSE_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 109
#define RSHMC804X_ATTR_OUTPUTS_FUSE_DELAY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 110
#define RSHMC804X_ATTR_OUTPUTS_FUSE_LINK                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 111
#define RSHMC804X_ATTR_OUTPUTS_FUSE_UNLINK                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 112
#define RSHMC804X_ATTR_OUTPUTS_FUSE_TRIPPED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 113
#define RSHMC804X_ATTR_OUTPUTS_RESET_OUTPUT_PROTECTION                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 94
#define RSHMC804X_ATTR_OUTPUTS_OVP_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 90
#define RSHMC804X_ATTR_OUTPUTS_OVP_LIMIT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 91
#define RSHMC804X_ATTR_OUTPUTS_OVP_TRIPPED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 114
#define RSHMC804X_ATTR_OUTPUTS_OVP_MODE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 115
#define RSHMC804X_ATTR_OUTPUTS_RESET_POWER_PROTECTION                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 98
#define RSHMC804X_ATTR_OUTPUTS_OPP_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 96
#define RSHMC804X_ATTR_OUTPUTS_OPP_LIMIT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 97
#define RSHMC804X_ATTR_OUTPUTS_OPP_TRIPPED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 116
#define RSHMC804X_ATTR_OUTPUTS_RAMP_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 117
#define RSHMC804X_ATTR_OUTPUTS_RAMP_DURATION                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 118
#define RSHMC804X_ATTR_OUTPUTS_AINPUT_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 100
#define RSHMC804X_ATTR_OUTPUTS_AINPUT_INPUT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 101
#define RSHMC804X_ATTR_OUTPUTS_AINPUT_MODE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 102
#define RSHMC804X_ATTR_OUTPUTS_AINPUT_THRESHOLD                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 103
#define RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 129
#define RSHMC804X_ATTR_OUTPUTS_MEAS_CHANNEL_LOG_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 150
#define RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_FORMAT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 130
#define RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_MODE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 131
#define RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_TIME                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 132
#define RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_COUNT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 133
#define RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_INTERVAL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 134
#define RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_TRIGGERED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 135
#define RSHMC804X_ATTR_OUTPUTS_MEAS_ENERGY_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 119
#define RSHMC804X_ATTR_OUTPUTS_MEAS_ENERGY_RESET                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 120
#define RSHMC804X_ATTR_OUTPUTS_SEQUENCE_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 139
#define RSHMC804X_ATTR_OUTPUTS_SEQUENCE_DELAY_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 140
#define RSHMC804X_ATTR_OUTPUTS_SEQUENCE_DELAY                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 141
#define RSHMC804X_ATTR_OUTPUTS_SEQUENCE_TRIGGERED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 153
#define RSHMC804X_ATTR_TRIGGER_SLOPE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 99
#define RSHMC804X_ATTR_HCOPY_IMAGE_FORMAT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 155
#define RSHMC804X_ATTR_HCOPY_IMAGE_WIDTH                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 156
#define RSHMC804X_ATTR_HCOPY_IMAGE_HEIGHT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 157
#define RSHMC804X_ATTR_SYSTEM_BEEPER_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 127
#define RSHMC804X_ATTR_SYSTEM_BEEP_START                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 128
#define RSHMC804X_ATTR_SYSTEM_ERROR                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 162
#define RSHMC804X_ATTR_SYSTEM_LOCAL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 163
#define RSHMC804X_ATTR_SYSTEM_REMOTE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 164
#define RSHMC804X_ATTR_SYSTEM_RWLOCK                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 165
#define RSHMC804X_ATTR_SYSTEM_VERSION                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 166
#define RSHMC804X_ATTR_SYSTEM_OPTIONS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 208
#define RSHMC804X_ATTR_SYSTEM_DEVICE_FOOT_PRINT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 209
#define RSHMC804X_ATTR_STATUS_PRESET                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 145
#define RSHMC804X_ATTR_LOGGING                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 189
#define RSHMC804X_ATTR_SELF_TEST_QUERY                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 190
#define RSHMC804X_ATTR_QUERY_OPC                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 192
#define RSHMC804X_ATTR_PROCESS_ALL_PREVIOUS_COMMANDS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 194

// rshmc804x_rngMeasurementType
#define RSHMC804X_VAL_MEASURE_VOLTAGE                                       0
#define RSHMC804X_VAL_MEASURE_CURRENT                                       1
#define RSHMC804X_VAL_MEASURE_POWER                                         2
#define RSHMC804X_VAL_MEASURE_ENERGY                                        3

// rshmc804x_rngTriggerSlope
#define RSHMC804X_VAL_TRIGGER_SLOPE_NEGATIVE                                0
#define RSHMC804X_VAL_TRIGGER_SLOPE_POSITIVE                                1

// rshmc804x_rngOvpMode
#define RSHMC804X_VAL_OVP_MODE_MEASURED                                     0
#define RSHMC804X_VAL_OVP_MODE_PROTECTED                                    1

// rshmc804x_rngAnalogInput
#define RSHMC804X_VAL_AINPUT_VOLTAGE                                        0
#define RSHMC804X_VAL_AINPUT_CURRENT                                        1

// rshmc804x_rngAnalogInputMode
#define RSHMC804X_VAL_AINPUT_MODE_LINEAR                                    0
#define RSHMC804X_VAL_AINPUT_MODE_STEP                                      1

// rshmc804x_rngLoggingFormat
#define RSHMC804X_VAL_LOGGING_FORMAT_CSV                                    0
#define RSHMC804X_VAL_LOGGING_FORMAT_TXT                                    1

// rshmc804x_rngLoggingMode
#define RSHMC804X_VAL_LOGGING_MODE_UNLIMITED                                0
#define RSHMC804X_VAL_LOGGING_MODE_COUNT                                    1
#define RSHMC804X_VAL_LOGGING_MODE_TIME                                     2

// rshmc804x_rngDevice
#define RSHMC804X_VAL_DEVICE_INT                                            0
#define RSHMC804X_VAL_DEVICE_EXT                                            1
#define RSHMC804X_VAL_DEVICE_DEF                                            2

// rshmc804x_rngRegisterGroup
#define RSHMC804X_VAL_REGISTER_GROUP_OPER                                   0
#define RSHMC804X_VAL_REGISTER_GROUP_QUES                                   1
#define RSHMC804X_VAL_REGISTER_GROUP_QUES_INST                              2
#define RSHMC804X_VAL_REGISTER_GROUP_QUES_INST_ISUM                         3

// rshmc804x_rngRegisterType
#define RSHMC804X_VAL_REGISTER_TYPE_COND                                    0
#define RSHMC804X_VAL_REGISTER_TYPE_EVEN                                    1
#define RSHMC804X_VAL_REGISTER_TYPE_ENAB                                    2

// rshmc804x_rngImageFormat
#define RSHMC804X_VAL_IMAGE_FORMAT_BMP                                      0
#define RSHMC804X_VAL_IMAGE_FORMAT_PNG                                      1

// rshmc804x_rngTriggerMode
#define RSHMC804X_VAL_TRIGGER_MODE_SINGLE                                   0
#define RSHMC804X_VAL_TRIGGER_MODE_RUN                                      1

// RsHmc804x_rngChannelFusion
#define RSHMC804X_VAL_CHANNEL_FUSION_OFF                                    0
#define RSHMC804X_VAL_CHANNEL_FUSION_SERIES                                 1
#define RSHMC804X_VAL_CHANNEL_FUSION_PARALLEL                               2

// RsHmc804x_rngOutputMode
#define RSHMC804X_VAL_OUTPUT_MODE_AUTO                                      0
#define RSHMC804X_VAL_OUTPUT_MODE_SINK                                      1
#define RSHMC804X_VAL_OUTPUT_MODE_SOURCE                                    2

// RsHmc804x_rngQueriedOutputMode
#define RSHMC804X_VAL_OUTPUT_MODE_OFF                                       3
#define RSHMC804X_VAL_OUTPUT_MODE_CV                                        4
#define RSHMC804X_VAL_OUTPUT_MODE_CC                                        5

//repcap Channel
#define RSHMC804X_REPCAP_CHANNEL_CH1                                        1
#define RSHMC804X_REPCAP_CHANNEL_CH2                                        2
#define RSHMC804X_REPCAP_CHANNEL_CH3                                        3
#define RSHMC804X_REPCAP_CHANNEL_CH4                                        4
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSHMC804X_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSHMC804X_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSHMC804X_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSHMC804X_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *------------------------ Attribute Value Defines -------------------------*
 ****************************************************************************/

    /*- Instrument specific attribute value definitions --------------------*/

#define RSHMC804X_VAL_STEP_SIZE_VOLTAGE 0
#define RSHMC804X_VAL_STEP_SIZE_CURRENT    1

#define RSHMC804X_VAL_RANGE_VOLTAGE        0
#define RSHMC804X_VAL_RANGE_CURRENT        1

#define RSHMC804X_VAL_FUSE_LINKING        0
#define RSHMC804X_VAL_FUSE_UNLINKING    1

#define RSHMC804X_VAL_FILE_OPER_SAVE    0
#define RSHMC804X_VAL_FILE_OPER_LOAD    1

/****************************************************************************
 *---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/
ViStatus _VI_FUNC rshmc804x_init(ViRsrc     resourceName,
                                 ViBoolean  IDQuery,
                                 ViBoolean  resetDevice,
                                 ViSession* instrSession);

ViStatus _VI_FUNC rshmc804x_InitWithOptions(ViRsrc     resourceName,
                                            ViBoolean  IDQuery,
                                            ViBoolean  resetDevice,
                                            ViString   optionString,
                                            ViSession* instrSession);

ViStatus _VI_FUNC rshmc804x_ConfigureOutput(ViSession instrSession,
                                            ViInt32   channel,
                                            ViReal64  voltage,
                                            ViReal64  current);

ViStatus _VI_FUNC rshmc804x_ConfigureStepSize(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   stepSizeType,
                                              ViReal64  stepSize);

ViStatus _VI_FUNC rshmc804x_ConfigureVoltageLevel(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViReal64  level);

ViStatus _VI_FUNC rshmc804x_ConfigureOVP(ViSession instrSession,
                                         ViInt32   channel,
                                         ViBoolean enabled,
                                         ViReal64  limit);

ViStatus _VI_FUNC rshmc804x_ConfigureOVPMode(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   OVPMode);

ViStatus _VI_FUNC rshmc804x_ConfigureOPP(ViSession instrSession,
                                         ViInt32   channel,
                                         ViBoolean enabled,
                                         ViReal64  limit);

ViStatus _VI_FUNC rshmc804x_ConfigureRamp(ViSession instrSession,
                                          ViInt32   channel,
                                          ViBoolean enabled,
                                          ViReal64  duration);

ViStatus _VI_FUNC rshmc804x_ConfigureAnalogInput(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViBoolean enabled,
                                                 ViInt32   input,
                                                 ViInt32   mode,
                                                 ViReal64  threshold);

ViStatus _VI_FUNC rshmc804x_ConfigureCurrentLimit(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViReal64  limit);

ViStatus _VI_FUNC rshmc804x_ConfigureOutputRange(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   rangeType,
                                                 ViReal64  range);

ViStatus _VI_FUNC rshmc804x_ConfigureOutputEnabled(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean enabled);

ViStatus _VI_FUNC rshmc804x_ConfigureChannelOnlyEnabled(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViBoolean enabled);

ViStatus _VI_FUNC rshmc804x_ConfigureMasterOutputEnabled(ViSession instrSession,
                                                         ViBoolean enabled);

ViStatus _VI_FUNC rshmc804x_QueryMaxCurrentLimit(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViReal64  voltageLevel,
                                                 ViReal64* maxCurrentLimit);

ViStatus _VI_FUNC rshmc804x_QueryMaxVoltageLevel(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViReal64  currentLimit,
                                                 ViReal64* maxVoltageLevel);

ViStatus _VI_FUNC rshmc804x_ConfigureChannelFusion(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   channelFusion);

ViStatus _VI_FUNC rshmc804x_ConfigureOutputMode(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   mode);

ViStatus _VI_FUNC rshmc804x_QueryOutputMode(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32*  mode);

ViStatus _VI_FUNC rshmc804x_ConfigureVoltmeterEnabled(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViBoolean enabled);

ViStatus _VI_FUNC rshmc804x_QueryVoltmeterValue(ViSession instrSession,
                                                ViInt32   channel,
                                                ViReal64* voltmeterValue);

ViStatus _VI_FUNC rshmc804x_ShowMessageBox(ViSession instrSession,
                                           ViString  text);

ViStatus _VI_FUNC rshmc804x_ClearMessageBox(ViSession instrSession);

ViStatus _VI_FUNC rshmc804x_ConfigureFuse(ViSession instrSession,
                                          ViInt32   channel,
                                          ViBoolean enabled,
                                          ViReal64  delay);

ViStatus _VI_FUNC rshmc804x_ConfigureFuseLinking(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   fuseLinkingType,
                                                 ViInt32   channelFuses);

ViStatus _VI_FUNC rshmc804x_ConfigureArbitrary(ViSession instrSession,
                                               ViInt32   channel,
                                               ViBoolean enabled,
                                               ViInt32   repetitions,
                                               ViInt32   endpoint);

ViStatus _VI_FUNC rshmc804x_ConfigureArbitraryFile(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViString  filename,
                                                   ViInt32   device,
                                                   ViInt32   operation);

ViStatus _VI_FUNC rshmc804x_ConfigureArbitraryTriggerMode(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   triggerMode);

ViStatus _VI_FUNC rshmc804x_WriteArbitraryData(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   numberOfValues,
                                               ViReal64  voltage[],
                                               ViReal64  current[],
                                               ViReal64  time[],
                                               ViInt32   interpolationMode[]);

ViStatus _VI_FUNC rshmc804x_ReadArbitraryData(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   timeOut_msec,
                                              ViInt32   arraySize,
                                              ViReal64  voltage[],
                                              ViReal64  current[],
                                              ViReal64  time[],
                                              ViInt32   interpolationMode[],
                                              ViInt32*  returnedValues);

ViStatus _VI_FUNC rshmc804x_ConfigureLoggingFile(ViSession instrSession,
                                                 ViString  filename,
                                                 ViInt32   device,
                                                 ViInt32   format);

ViStatus _VI_FUNC rshmc804x_ReadFileFromInstrument(ViSession instrSession,
                                                   ViString  filename,
                                                   ViInt32   device,
                                                   ViInt32   bufferSize,
                                                   ViChar    fileContent[]);

ViStatus _VI_FUNC rshmc804x_Delete(ViSession instrSession,
                                   ViString  filename,
                                   ViInt32   device);

ViStatus _VI_FUNC rshmc804x_ReadDataPoints(ViSession instrSession,
                                           ViString  filename,
                                           ViInt32   device,
                                           ViInt32*  points);

ViStatus _VI_FUNC rshmc804x_ReadDataCatalog(ViSession instrSession,
                                            ViInt32   device,
                                            ViInt32   bufferSize,
                                            ViChar    catalog[]);

ViStatus _VI_FUNC rshmc804x_ConfigureLogging(ViSession instrSession,
                                             ViBoolean enabled,
                                             ViInt32   mode,
                                             ViInt32   count,
                                             ViReal64  time,
                                             ViReal64  interval);

ViStatus _VI_FUNC rshmc804x_ConfigureLoggingTriggered(ViSession instrSession,
                                                      ViBoolean triggered);

ViStatus _VI_FUNC rshmc804x_ConfigureChannelLoggingState(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViBoolean state);

ViStatus _VI_FUNC rshmc804x_ConfigureEnergyState(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViBoolean energyState);

ViStatus _VI_FUNC rshmc804x_ConfigureSequenceState(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean state);

ViStatus _VI_FUNC rshmc804x_ConfigureSequenceDelay(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean enabled,
                                                   ViReal64  delay);

ViStatus _VI_FUNC rshmc804x_ConfigureTriggerSlope(ViSession instrSession,
                                                  ViInt32   triggerSlope);

ViStatus _VI_FUNC rshmc804x_QueryTriggerSlope(ViSession instrSession,
                                              ViInt32*  triggerSlope);

ViStatus _VI_FUNC rshmc804x_QueryOVPTripped(ViSession  instrSession,
                                            ViInt32    channel,
                                            ViBoolean* tripped);

ViStatus _VI_FUNC rshmc804x_QueryOPPTripped(ViSession  instrSession,
                                            ViInt32    channel,
                                            ViBoolean* tripped);

ViStatus _VI_FUNC rshmc804x_QueryFuseTripped(ViSession  instrSession,
                                             ViInt32    channel,
                                             ViBoolean* tripped);

ViStatus _VI_FUNC rshmc804x_QueryArbitraryTriggered(ViSession  instrSession,
                                                    ViInt32    channel,
                                                    ViBoolean* triggered);

ViStatus _VI_FUNC rshmc804x_QuerySequenceTriggered(ViSession  instrSession,
                                                   ViInt32    channel,
                                                   ViBoolean* triggered);

ViStatus _VI_FUNC rshmc804x_QueryLoggingTriggered(ViSession  instrSession,
                                                  ViBoolean* triggered);

ViStatus _VI_FUNC rshmc804x_ResetOutputProtection(ViSession instrSession,
                                                  ViInt32   channel);

ViStatus _VI_FUNC rshmc804x_ResetPowerProtection(ViSession instrSession,
                                                 ViInt32   channel);

ViStatus _VI_FUNC rshmc804x_ResetEnergyCounter(ViSession instrSession,
                                               ViInt32   channel);

ViStatus _VI_FUNC rshmc804x_ResetArbitrary(ViSession instrSession,
                                           ViInt32   channel);

ViStatus _VI_FUNC rshmc804x_ResetStepSize(ViSession instrSession,
                                          ViInt32   channel,
                                          ViInt32   stepSizeType);

ViStatus _VI_FUNC rshmc804x_Measure(ViSession instrSession,
                                    ViInt32   channel,
                                    ViInt32   measurementType,
                                    ViReal64* measurement);

ViStatus _VI_FUNC rshmc804x_SetAttributeViInt32(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViUInt32  attributeID,
                                                ViInt32   attributeValue);

ViStatus _VI_FUNC rshmc804x_SetAttributeViReal64(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeID,
                                                 ViReal64  attributeValue);

ViStatus _VI_FUNC rshmc804x_SetAttributeViString(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeID,
                                                 ViString  attributeValue);

ViStatus _VI_FUNC rshmc804x_SetAttributeViBoolean(ViSession instrSession,
                                                  ViString  repeatedCapabilityName,
                                                  ViUInt32  attributeID,
                                                  ViBoolean attributeValue);

ViStatus _VI_FUNC rshmc804x_GetAttributeRepCapName(ViSession instrSession,
                                                   ViUInt32  attributeID,
                                                   ViString  repCapNameID,
                                                   ViInt32   index,
                                                   ViInt32   bufferSize,
                                                   ViChar    repCapName[]);

ViStatus _VI_FUNC rshmc804x_GetAttributeRepeatedCapabilityIds(
                            ViSession instrSession,
                            ViUInt32  attributeID,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIds[]);

ViStatus _VI_FUNC rshmc804x_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViUInt32  attributeID,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdNames[]);

ViStatus _VI_FUNC rshmc804x_GetAttributeViInt32(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViUInt32  attributeID,
                                                ViInt32*  attributeValue);

ViStatus _VI_FUNC rshmc804x_GetAttributeViReal64(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeID,
                                                 ViReal64* attributeValue);

ViStatus _VI_FUNC rshmc804x_GetAttributeViString(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeID,
                                                 ViInt32   bufferSize,
                                                 ViChar    attributeValue[]);

ViStatus _VI_FUNC rshmc804x_GetAttributeViBoolean(ViSession  instrSession,
                                                  ViString   repeatedCapabilityName,
                                                  ViUInt32   attributeID,
                                                  ViBoolean* attributeValue);

ViStatus _VI_FUNC rshmc804x_reset(ViSession instrSession);

ViStatus _VI_FUNC rshmc804x_self_test(ViSession instrSession,
                                      ViInt16*  selfTestResult,
                                      ViChar    selfTestMessage[]);

ViStatus _VI_FUNC rshmc804x_revision_query(ViSession instrSession,
                                           ViChar    instrumentDriverRevision[],
                                           ViChar    firmwareRevision[]);

ViStatus _VI_FUNC rshmc804x_BeeperState(ViSession instrSession,
                                        ViBoolean beeperState);

ViStatus _VI_FUNC rshmc804x_StartBeep(ViSession instrSession);

ViStatus _VI_FUNC rshmc804x_SystemLocal(ViSession instrSession);

ViStatus _VI_FUNC rshmc804x_SystemRemote(ViSession instrSession);

ViStatus _VI_FUNC rshmc804x_SystemRWLock(ViSession instrSession);

ViStatus _VI_FUNC rshmc804x_SystemVersion(ViSession instrSession,
                                          ViInt32   length,
                                          ViChar    systemVersion[]);

ViStatus _VI_FUNC rshmc804x_SystemOptions(ViSession instrSession,
                                          ViInt32   length,
                                          ViChar    systemOptions[]);

ViStatus _VI_FUNC rshmc804x_SystemDeviceFootPrint(ViSession instrSession,
                                                  ViInt32   length,
                                                  ViChar    systemDeviceFootPrint[]);

ViStatus _VI_FUNC rshmc804x_error_query(ViSession instrSession,
                                        ViInt32*  errorCode,
                                        ViChar    errorMessage[]);

ViStatus _VI_FUNC rshmc804x_error_message(ViSession instrSession,
                                          ViStatus  errorCode,
                                          ViChar    errorMessage[]);

ViStatus _VI_FUNC rshmc804x_SetStatusRegister(ViSession instrSession,
                                              ViInt32   registerGroup,
                                              ViInt32   ISummary,
                                              ViInt32   registerType,
                                              ViInt32   registerValue);

ViStatus _VI_FUNC rshmc804x_GetStatusRegister(ViSession instrSession,
                                              ViInt32   registerGroup,
                                              ViInt32   ISummary,
                                              ViInt32   registerType,
                                              ViInt32*  registerValue);

ViStatus _VI_FUNC rshmc804x_StatusPreset(ViSession instrSession);

ViStatus _VI_FUNC rshmc804x_ClearStatus(ViSession instrSession);

ViStatus _VI_FUNC rshmc804x_IDQueryResponse(ViSession instrSession,
                                            ViChar    IDQueryResponse[]);

ViStatus _VI_FUNC rshmc804x_ProcessAllPreviousCommands(ViSession instrSession);

ViStatus _VI_FUNC rshmc804x_QueryOPC(ViSession instrSession,
                                     ViInt32*  OPC);

ViStatus _VI_FUNC rshmc804x_ConfigureErrorChecking(ViSession instrSession,
                                                   ViBoolean optionChecking,
                                                   ViBoolean rangeChecking,
                                                   ViBoolean statusChecking);

ViStatus _VI_FUNC rshmc804x_SetVISATimeout(ViSession instrSession,
                                           ViUInt32  VISATimeout);

ViStatus _VI_FUNC rshmc804x_GetVISATimeout(ViSession instrSession,
                                           ViUInt32* VISATimeout);

ViStatus _VI_FUNC rshmc804x_SetOPCTimeout(ViSession instrSession,
                                          ViInt32   timeout);

ViStatus _VI_FUNC rshmc804x_GetOPCTimeout(ViSession instrSession,
                                          ViInt32*  timeout);

ViStatus _VI_FUNC rshmc804x_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                        ViBoolean autoSystErrQuery);

ViStatus _VI_FUNC rshmc804x_ConfigureMultiThreadLocking(ViSession instrSession,
                                                        ViBoolean multiThreadLocking);

ViStatus _VI_FUNC rshmc804x_GetError(ViSession instrSession,
                                     ViStatus* code,
                                     ViInt32   bufferSize,
                                     ViChar    description[]);

ViStatus _VI_FUNC rshmc804x_ClearError(ViSession instrSession);

ViStatus _VI_FUNC rshmc804x_WriteInstrData(ViSession instrSession,
                                           ViString  writeBuffer);

ViStatus _VI_FUNC rshmc804x_QueryViBoolean(ViSession  instrSession,
                                           ViString   cmd,
                                           ViBoolean* value);

ViStatus _VI_FUNC rshmc804x_QueryViInt32(ViSession instrSession,
                                         ViString  cmd,
                                         ViInt32*  value);

ViStatus _VI_FUNC rshmc804x_QueryViReal64(ViSession instrSession,
                                          ViString  cmd,
                                          ViReal64* value);

ViStatus _VI_FUNC rshmc804x_QueryViString(ViSession instrSession,
                                          ViString  cmd,
                                          ViInt32   bufferSize,
                                          ViChar    value[]);

ViStatus _VI_FUNC rshmc804x_ImageSnapshot(ViSession instrSession,
                                          ViString  destination);

ViStatus _VI_FUNC rshmc804x_ConfigureImageFormat(ViSession instrSession,
                                                 ViInt32   imageFormat);

ViStatus _VI_FUNC rshmc804x_QueryImageSize(ViSession instrSession,
                                           ViInt32*  imageWidth,
                                           ViInt32*  imageHeight);

ViStatus _VI_FUNC rshmc804x_close(ViSession instrSession);

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSHMC804X_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSHMC804X_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSHMC804X_ERROR_INCORRECT_STATE        (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSHMC804X_ERROR_INCORRECT_RESPONSE     (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSHMC804X_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSHMC804X_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSHMC804X_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSHMC804X_ERRMSG_INCORRECT_STATE     "Instrument in incorrect state"
#define RSHMC804X_ERRMSG_INCORRECT_RESPONSE      "Response incorrect; report problem to R&S support!"
#define RSHMC804X_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSHMC804X_ERROR_CODES_AND_MSGS \
    {RSHMC804X_WARN_MEASURE_UNCALIBRATED,  RSHMC804X_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSHMC804X_WARN_OVER_RANGE,            RSHMC804X_WARNMSG_OVER_RANGE},\
    {RSHMC804X_ERROR_INCORRECT_STATE ,   RSHMC804X_ERRMSG_INCORRECT_STATE },\
    {RSHMC804X_ERROR_INCORRECT_RESPONSE,     RSHMC804X_ERRMSG_INCORRECT_RESPONSE},\
    {RSHMC804X_ERROR_MAX_TIME_EXCEEDED,    RSHMC804X_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSHMC804X_HEADER */
